function theta=e2m_bernoulli(p,q,theta0,visu)

% E2M algorithm for the Bernoulli model with uncertain data

% INPUT
% p: (n,1) vector of plausibilities pl_i(0)
% q: (n,1) vector of plausibilities pl_i(1)
% theta0: initial value of parameter theta 
% visu : 1 to display intermediate results, 0 otherwise
%
% OUTPUT
% theta: estimated value of the parameter after convergence
%
% References:
%
% T. Denoeux. Maximum likelihood from evidential data: an extension of the 
% EM algorithm. In C. Borgelt et al. (Eds), Combining soft computing and 
% statistical methods in data analysis (Proceedings of SMPS 2010, Oviedo, Spain, 
% September 28 - October 1, 2010), Advances in Intelligent and Soft Computing, 
% pages 181-188, Springer, 2010.
%
% T. Denoeux, Maximum likelihood estimation from Uncertain Data in the Belief
% Function Framework, IEEE Transactions on Knowledge and Data Engineering
% (to appear), 2011.
%
% Written by Thierry Denoeux, 11 Augut 2011.

theta=theta0;
L0=prod((1-theta)*p+theta*q);
if visu, disp([theta 1000*L0]); end;

go_on=1;
k=0;

while go_on,
    k=k+1;
    xi=theta*q./((1-theta)*p+theta*q);
    theta=mean(xi);
    L1=prod((1-theta)*p+theta*q);
    if (L1-L0)./abs(L0) < 1e-6,  % stopping criterion (may be changed)
        go_on=0;
    end
    L0=L1;
    if visu, disp([k theta 1000*L0]); end;
end;
